@@ -401,7 +401,8 @@ class MessageCoexecutionPodJobClient(BaseMessageJobClient):
         self.pulsar_container_image = destination_params.get("pulsar_container_image", "galaxy/pulsar-pod-staging:0.13.0")
         self._default_pull_policy = pull_policy(destination_params)
 
-    def launch(
+    # FIXME: pulsar/client/client.py:404:5: C901 'MessageCoexecutionPodJobClient.launch' is too complex (13)
+    def launch(  # noqa: C901
         self,
         command_line,
         dependencies_description=None,
@@ -426,9 +427,18 @@ class MessageCoexecutionPodJobClient(BaseMessageJobClient):
             container = container_info.get("container_id")
             guest_ports = container_info.get("guest_ports")
 
+        manager_name = self.client_manager.manager_name
         manager_type = "coexecution" if container is not None else "unqueued"
         if "manager" not in pulsar_app_config and "managers" not in pulsar_app_config:
-            pulsar_app_config["manager"] = {"type": manager_type}
+            pulsar_app_config["managers"] = {manager_name: {"type": manager_type}}
+        elif "manager" in pulsar_app_config and manager_name != '_default_':
+            log.warning(
+                "'manager' set in app config but client has non-default manager '%s', this will cause communication"
+                " failures, remove `manager` from app or client config to fix", manager_name)
+
+        manager_args = []
+        if manager_name != "_default_":
+            manager_args = ["--manager", manager_name]
 
         using_dependencies = container is None and dependencies_description is not None
         if using_dependencies and "dependency_resolution" not in pulsar_app_config:
@@ -459,6 +469,7 @@ class MessageCoexecutionPodJobClient(BaseMessageJobClient):
         params = self.destination_params
 
         pulsar_container_image = self.pulsar_container_image
+        pulsar_container_resources = self._pulsar_container_resources(params)
 
         job_directory = self.job_directory
 
@@ -472,11 +483,14 @@ class MessageCoexecutionPodJobClient(BaseMessageJobClient):
             "name": "pulsar-container",
             "image": pulsar_container_image,
             "command": ["pulsar-submit"],
-            "args": ["--base64", base64_message, "--app_conf_base64", base64_app_conf],
+            "args": ["--base64", base64_message, "--app_conf_base64", base64_app_conf] + manager_args,
             "workingDir": "/",
             "volumeMounts": volume_mounts,
         }
+        if pulsar_container_resources:
+            pulsar_container_dict["resources"] = pulsar_container_resources
         tool_container_image = container
+        tool_container_resources = self._tool_container_resources(params)
         container_dicts = [pulsar_container_dict]
         if container:
             command = TOOL_EXECUTION_CONTAINER_COMMAND_TEMPLATE % job_directory.job_directory
@@ -488,6 +502,8 @@ class MessageCoexecutionPodJobClient(BaseMessageJobClient):
                 "workingDir": "/",
                 "volumeMounts": volume_mounts,
             }
+            if tool_container_resources:
+                tool_container_spec["resources"] = tool_container_resources
             if guest_ports:
                 tool_container_spec["ports"] = [{"containerPort": int(p)} for p in guest_ports]
             container_dicts.append(tool_container_spec)
@@ -506,6 +522,8 @@ class MessageCoexecutionPodJobClient(BaseMessageJobClient):
             }
         }
         spec = {"template": template}
+        if "k8s_walltime_limit" in params:
+            spec["activeDeadlineSeconds"] = int(params["k8s_walltime_limit"])
         k8s_job_obj = job_object_dict(params, job_name, spec)
         pykube_client = self._pykube_client
         job = Job(pykube_client, k8s_job_obj)
@@ -546,6 +564,26 @@ class MessageCoexecutionPodJobClient(BaseMessageJobClient):
         job_name = produce_unique_k8s_job_name(app_prefix="pulsar", job_id=job_id, instance_id=self.instance_id)
         return job_name
 
+    def _pulsar_container_resources(self, params):
+        return self._container_resources(params, container='pulsar')
+
+    def _tool_container_resources(self, params):
+        return self._container_resources(params, container='tool')
+
+    def _container_resources(self, params, container=None):
+        resources = {}
+        for resource_param in ('requests_cpu', 'requests_memory', 'limits_cpu', 'limits_memory'):
+            subkey, resource = resource_param.split('_', 1)
+            if resource_param in params:
+                if subkey not in resources:
+                    resources[subkey] = {}
+                resources[subkey][resource] = params[resource_param]
+            if container is not None and container + '_' + resource_param in params:
+                if subkey not in resources:
+                    resources[subkey] = {}
+                resources[subkey][resource] = params[container + '_' + resource_param]
+        return resources
+
 
 class InputCachingJobClient(JobClient):
     """
